using System;
using System.Collections.Generic;
using System.Text;

namespace ManualCheckVertex
{
    class Program
    {
        // stream writer for log file (dump of console stream)
        static System.IO.StreamWriter gLog = null;

        //___________________________________________________________________________//
        static void WriteLine(string str)
        {
            Console.WriteLine(str);
            if (gLog != null) gLog.WriteLine(str);
        }
        //___________________________________________________________________________//
        static void ShowExplanation()
        {
            Console.WriteLine("usage: ManualCheckVertex <input TSR> <vertex_ID> <sheet_ID>");
        }
        //___________________________________________________________________________//
        static void ShowIntercalibration(SySal.TotalScan.Volume Vol, int layerID)
        {
            SySal.TotalScan.Layer lay = Vol.Layers[layerID];
            SySal.DAQSystem.Scanning.IntercalibrationInfo Inv
                = new SySal.DAQSystem.Scanning.IntercalibrationInfo();

            Inv.RX = Vol.RefCenter.X + lay.AlignData.TranslationX;
            Inv.RY = Vol.RefCenter.Y + lay.AlignData.TranslationY;
            double iden = 1.0 / (lay.AlignData.AffineMatrixXX * lay.AlignData.AffineMatrixYY - lay.AlignData.AffineMatrixXY * lay.AlignData.AffineMatrixYX);
            Inv.MXX = lay.AlignData.AffineMatrixYY * iden;
            Inv.MXY = -lay.AlignData.AffineMatrixXY * iden;
            Inv.MYX = -lay.AlignData.AffineMatrixYX * iden;
            Inv.MYY = lay.AlignData.AffineMatrixXX * iden;
            Inv.TX = -lay.AlignData.TranslationX;
            Inv.TY = -lay.AlignData.TranslationY;

            WriteLine("INTERCALIBRATION INFO\r\n"
                + "Inv.RX  " + Inv.RX.ToString() + "\r\n"
                + "Inv.RY  " + Inv.RY.ToString() + "\r\n"
                + "Inv.MXX " + Inv.MXX.ToString() + "\r\n"
                + "Inv.MXY " + Inv.MXY.ToString() + "\r\n"
                + "Inv.MYX " + Inv.MYX.ToString() + "\r\n"
                + "Inv.MYY " + Inv.MYY.ToString() + "\r\n"
                + "Inv.TX  " + Inv.TX.ToString() + "\r\n"
                + "Inv.TY  " + Inv.TY.ToString() + "\r\n"
                + "\r\n");

            /* 
            // HOW TO TRANSFORM A TRACK (BACK TO ITS ORIGINAL COORDINATES)
            SySal.Tracking.MIPEmulsionTrackInfo info = trk.Info;
            double ipx = Inv.MXX * (info.Intercept.X - Inv.RX) + Inv.MXY * (info.Intercept.Y - Inv.RY) + Inv.TX + Inv.RX;
            double ipy = Inv.MYX * (info.Intercept.X - Inv.RX) + Inv.MYY * (info.Intercept.Y - Inv.RY) + Inv.TY + Inv.RY;
            double isx = Inv.MXX * info.Slope.X + Inv.MXY * info.Slope.Y;
            double isy = Inv.MYX * info.Slope.X + Inv.MYY * info.Slope.Y;
            */
        }
        //___________________________________________________________________________//
        static string ToOriginal(int id, SySal.Tracking.MIPEmulsionTrackInfo info, SySal.TotalScan.Layer l)
        {
            info.Intercept = l.ToOriginalPoint(info.Intercept);
            info.Slope = l.ToOriginalSlope(info.Slope);

            string Text = id.ToString()
                           + "  " + info.Intercept.X.ToString("F1")
                           + "  " + info.Intercept.Y.ToString("F1")
                           + "  " + info.Slope.X.ToString("F5")
                           + "  " + info.Slope.Y.ToString("F5")
                           + "  " + info.Count.ToString();
            return Text;
        }
        //___________________________________________________________________________//
        static string ProjectToLayer(SySal.TotalScan.Track trk, SySal.TotalScan.Layer layer)
        {
            SySal.Tracking.MIPEmulsionTrackInfo info = trk.Fit(layer.Id, layer.RefCenter.Z);

            info.Intercept = layer.ToOriginalPoint(info.Intercept);
            info.Slope = layer.ToOriginalSlope(info.Slope);

            string Text = trk.Id.ToString()
                           + "  " + info.Intercept.X.ToString("F1")
                           + "  " + info.Intercept.Y.ToString("F1")
                           + "  " + info.Slope.X.ToString("F5")
                           + "  " + info.Slope.Y.ToString("F5")
                           + "  " + "0";
            return Text;
        }

        //___________________________________________________________________________//
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]

        static void Main(string[] args)
        {
            // Parse arguments
            string tsr_fname;
            int vertexID, sheetID;
            try
            {
                if (args.Length != 3) throw new Exception("Invalid arguments");
                tsr_fname = args[0];
                vertexID = Convert.ToInt32(args[1]);
                sheetID = Convert.ToInt32(args[2]);
            }
            catch (Exception x)
            {
                Console.WriteLine(x.Message);
                ShowExplanation();
                return;
            }

            // Store CultureInfo for the current culture. Note that the original culture
            // can be set and retrieved on the current thread object.
            System.Threading.Thread thread = System.Threading.Thread.CurrentThread;
            System.Globalization.CultureInfo originalCulture = thread.CurrentCulture;
            // Set the culture to en-US.
            thread.CurrentCulture = new System.Globalization.CultureInfo("en-US");

            //
            System.IO.StreamWriter sw = null;
            SySal.TotalScan.Volume vol = null;

            try
            {
                // OPEN A FILE TO WRITE VERTEX SUMMARY
                gLog = new System.IO.StreamWriter("vtx_" + vertexID.ToString() + ".txt");

                // OPEN A FILE TO WRITE TRACKS FOR MANUAL CHECK
                sw = new System.IO.StreamWriter("vtx_" + vertexID.ToString() + "pl_" + sheetID.ToString() + ".txt");
                sw.WriteLine("ID\tPX\tPY\tSX\tSY\tN");

                // OPEN TSR 
                WriteLine("open " + tsr_fname);
                vol = (SySal.TotalScan.Volume)SySal.OperaPersistence.Restore(tsr_fname, typeof(SySal.TotalScan.Volume));

                WriteLine("layers   :" + vol.Layers.Length.ToString());
                WriteLine("tracks   :" + vol.Tracks.Length.ToString());
                WriteLine("vertices :" + vol.Vertices.Length.ToString());

                // SELECT LAYER
                int layerId = -999;
                int nlayers = vol.Layers.Length;
                for (int i = 0; i < nlayers; i++)
                {
                    SySal.TotalScan.Layer lay = vol.Layers[i];
                    WriteLine(
                          " layer : " + lay.Id.ToString("D2")
                        + " sheet : " + lay.SheetId.ToString("D2")
                        + " Z up : " + lay.UpstreamZ.ToString("F1")
                        + " RefCenter.Z : " + lay.RefCenter.Z.ToString("F1")
                        + " Z down : " + lay.DownstreamZ.ToString("F1")
                        );
                    if (lay.SheetId == sheetID)
                        layerId = lay.Id;
                }
                if (layerId == -999)
                    throw (new Exception(" sheet ID " + sheetID.ToString() + " not included in this volume"));
                WriteLine(" selected layer: " + layerId.ToString() + " (sheetID: " + sheetID.ToString() + ")\r\n");
                SySal.TotalScan.Layer selected_layer = vol.Layers[layerId];

                // SHOW INTERCALIBRATION  
                ShowIntercalibration(vol, layerId);

                // GET VERTEX
                if (vertexID < 0 || vertexID > vol.Vertices.Length)
                    throw (new Exception(" vertex ID " + vertexID.ToString() + " out of range"));
                SySal.TotalScan.Vertex vtx = vol.Vertices[vertexID];
                WriteLine(vtx.ToString() + "\r\n");

                // GET TRACKS
                for (int i = 0; i < vtx.Length; i++)
                {
                    SySal.TotalScan.Track trk = vtx[i];
                    WriteLine(" - - - - - - - - - - - - - - - - - - \r\n" + trk.ToString() + "\r\n");
                    bool found = false;
                    for (int j = 0; j < trk.Length; j++)
                    {
                        WriteLine(trk[j].ToString() + "\r\n");
                        if (trk[j].LayerOwner.SheetId == sheetID)
                        {
                            sw.WriteLine(ToOriginal(trk.Id, trk[j].Info, selected_layer));
                            found = true;
                            continue;
                        }
                    }
                    if (!found)
                        sw.WriteLine(ProjectToLayer(trk, selected_layer));
                }

                sw.Flush();
                gLog.Flush();
            }
            catch (Exception x)
            {
                WriteLine("Unrecoverable error:\n" + x.Message);
            }
            finally
            {
                if (sw != null) sw.Close();
                if (gLog != null) gLog.Close();
            }
        }
        //___________________________________________________________________________//
    }
}
